/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ITelemetryLoggerExt } from "@fluidframework/telemetry-utils";
import { IFluidHandle } from "@fluidframework/core-interfaces";
import { IFluidSerializer } from "@fluidframework/shared-object-base";
import { IChannelStorageService } from "@fluidframework/datastore-definitions";
import { ISummaryTreeWithStats } from "@fluidframework/runtime-definitions";
import { PropertySet } from "./properties";
import { JsonSegmentSpecs, MergeTreeChunkV1 } from "./snapshotChunks";
import { MergeTree } from "./mergeTree";
export declare class SnapshotV1 {
    mergeTree: MergeTree;
    private readonly getLongClientId;
    filename?: string | undefined;
    onCompletion?: (() => void) | undefined;
    static readonly chunkSize: number;
    private readonly header;
    private readonly segments;
    private readonly segmentLengths;
    private readonly attributionCollections;
    private readonly logger;
    private readonly chunkSize;
    constructor(mergeTree: MergeTree, logger: ITelemetryLoggerExt, getLongClientId: (id: number) => string, filename?: string | undefined, onCompletion?: (() => void) | undefined);
    private getSeqLengthSegs;
    /**
     * Emits the snapshot to an ISummarizeResult. If provided the optional IFluidSerializer will be used when
     * serializing the summary data rather than JSON.stringify.
     */
    emit(serializer: IFluidSerializer, bind: IFluidHandle): ISummaryTreeWithStats;
    extractSync(): JsonSegmentSpecs[];
    static loadChunk(storage: IChannelStorageService, path: string, logger: ITelemetryLoggerExt, options: PropertySet | undefined, serializer?: IFluidSerializer): Promise<MergeTreeChunkV1>;
    static processChunk(path: string, chunk: string, logger: ITelemetryLoggerExt, options: PropertySet | undefined, serializer?: IFluidSerializer): MergeTreeChunkV1;
}
//# sourceMappingURL=snapshotV1.d.ts.map